/**********************************************************************
 *
 * gdbmclass.h -- 
 * Copyright (C) 1999-2008  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/


#ifndef GDBMCLASS_H
#define GDBMCLASS_H


#include "gsdlconf.h"
#include "text_t.h"
#include "infodbclass.h"

#ifdef __WIN32__

#ifdef __cplusplus
  extern "C" {
#endif
#include "autoconf.h"
#include "systems.h"
#include "gdbmconst.h"
#include "gdbm.h"
#ifdef __cplusplus
  }
#endif

#else
#include <gdbm.h>
#endif


class gdbmclass
{
public:
  gdbmclass() {gdbmfile = NULL; logout = &cerr;};
  ~gdbmclass();
  
  // returns true if opened
  bool opendatabase (const text_t &filename, int mode, int num_retrys, 
		     bool need_filelock);
  void closedatabase ();

  // replaces the .c, .p, .n, .l syntax (child, parent, next, previous) 
  // it expects child, parent, etc. to exist if syntax has been used
  // so you should test before using
  text_t translate_OID (const text_t &OID, infodbclass &info);

  // returns true on success
  bool getinfo (const text_t& key, infodbclass &info);
  void setlogout (ostream *thelogout) {logout = thelogout;}

  // returns true if exists
  bool exists (const text_t& key);

  // returns true on success
  bool setinfo (const text_t &key, const infodbclass &info);
  // returns true on success
  bool setinfo (const text_t &key, const text_t &data);

  void deletekey (const text_t &key);

  // getfirstkey and getnextkey are used for traversing the database
  // no insertions or deletions should be carried out while traversing
  // the database. when there are no keys left to visit in the database
  // an empty string is returned.
  text_t getfirstkey ();
  text_t getnextkey (const text_t &key);

    // returns true on success
  bool getkeydata (const text_t& key, text_t &data);

protected:
  text_t openfile;
  GDBM_FILE gdbmfile;
  ostream *logout;

  void get_first_child (text_t &OID, infodbclass &info);
  void get_last_child (text_t &OID, infodbclass &info);
  void get_next_sibling (text_t &OID, infodbclass &info);
  void get_previous_sibling (text_t &OID, infodbclass &info);


  // returns true on success
  bool getinfoline (text_t::iterator &here, text_t::iterator end, 
		   text_t &key, text_t &value);
};


#endif
